@@ -1,4 +1,3 @@
-from math import sqrt
 from time import time
 
 from twisted.internet.interfaces import IPushProducer
@@ -8,24 +7,18 @@ from twisted.python import log
 from zope.interface import implements
 
 from bravo.config import configuration
-from bravo.entity import Pickup, Player
+from bravo.entity import entities
 from bravo.ibravo import IAuthenticator, ISeason, ITerrainGenerator
+from bravo.location import Location
 from bravo.packets import make_packet
-from bravo.plugin import retrieve_named_plugins
-from bravo.protocols.beta import BravoProtocol
+from bravo.plugin import retrieve_named_plugins, retrieve_sorted_plugins
+from bravo.protocols.beta import BannedProtocol, BravoProtocol
 from bravo.utilities import chat_name, sanitize_chat
 from bravo.world import World
 
-from bravo.plugin import retrieve_plugins
-
 (STATE_UNAUTHENTICATED, STATE_CHALLENGED, STATE_AUTHENTICATED,
     STATE_LOCATED) = range(4)
 
-entities_by_name = {
-    "Player": Player,
-    "Pickup": Pickup,
-}
-
 class BravoFactory(Factory):
     """
     A ``Factory`` that creates ``BravoProtocol`` objects when connected to.
@@ -57,33 +50,35 @@ class BravoFactory(Factory):
         log.msg("Initializing factory for world '%s'..." % name)
 
         self.name = name
-        self.port = configuration.getint("world %s" % name, "port")
-        if configuration.has_option("world %s" % name, "host"):
-            self.interface = configuration.get("world %s" % name, "host")
+        self.config_name = "world %s" % name
 
-        world_folder = configuration.get("world %s" % name, "path")
-        self.world = World(world_folder)
+        self.port = configuration.getint(self.config_name, "port")
+        self.interface = configuration.getdefault(self.config_name, "host",
+            "")
+
+        self.world = World(name)
         self.world.factory = self
-        if configuration.getboolean("world %s" % name, "perm_cache"):
-            self.world.enable_cache()
+        if configuration.has_option(self.config_name, "perm_cache"):
+            cache_level = configuration.getint(self.config_name, "perm_cache")
+            self.world.enable_cache(cache_level)
 
         self.protocols = dict()
 
         self.eid = 1
-        self.entities = set()
 
+        self.time = self.world.time
         self.time_loop = LoopingCall(self.update_time)
         self.time_loop.start(2)
 
-        authenticator = configuration.get("world %s" % name, "authenticator")
+        authenticator = configuration.get(self.config_name, "authenticator")
         selected = retrieve_named_plugins(IAuthenticator, [authenticator])[0]
 
         log.msg("Using authenticator %s" % selected.name)
         self.handshake_hook = selected.handshake
         self.login_hook = selected.login
 
-        generators = configuration.getlist("bravo", "generators")
-        generators = retrieve_named_plugins(ITerrainGenerator, generators)
+        generators = configuration.getlist(self.config_name, "generators")
+        generators = retrieve_sorted_plugins(ITerrainGenerator, generators)
 
         log.msg("Using generators %s" % ", ".join(i.name for i in generators))
         self.world.pipeline = generators
@@ -92,17 +87,86 @@ class BravoFactory(Factory):
 
         log.msg("Factory successfully initialized for world '%s'!" % name)
 
+    def buildProtocol(self, addr):
+        """
+        Create a protocol.
+
+        This overriden method provides early player entity registration, as a
+        solution to the username/entity race that occurs on login.
+        """
+
+        banned = self.world.serializer.load_plugin_data("banned_ips")
+
+        for ip in banned.split():
+            if addr.host == ip:
+                # Use BannedProtocol with extreme prejudice.
+                log.msg("Kicking banned IP %s" % addr)
+                p = BannedProtocol()
+                p.factory = self
+                return p
+
+        log.msg("Starting connection for %s" % addr)
+        p = self.protocol(self.name)
+        p.factory = self
+
+        self.register_entity(p)
+
+        return p
+
     def create_entity(self, x, y, z, name, **kwargs):
-        self.eid += 1
-        entity = entities_by_name[name](self.eid, **kwargs)
-        entity.location.x = x
-        entity.location.y = y
-        entity.location.z = z
-        self.entities.add(entity)
+        """
+        Spawn an entirely new entity.
+
+        Handles entity registration as well as instantiation.
+        """
+
+        location = Location()
+        location.x = x
+        location.y = y
+        location.z = z
+        entity = entities[name](eid=0, location=location, **kwargs)
+
+        self.register_entity(entity)
+
+        bigx = entity.location.x // 16
+        bigz = entity.location.z // 16
+
+        chunk = self.world.load_chunk(bigx, bigz)
+        chunk.entities.add(entity)
+
+        log.msg("Created entity %s" % entity)
+
         return entity
 
+    def register_entity(self, entity):
+        """
+        Registers an entity with this factory.
+
+        Registration is perhaps too fancy of a name; this method merely makes
+        sure that the entity has a unique and usable entity ID.
+        """
+
+        if not entity.eid:
+            self.eid += 1
+            entity.eid = self.eid
+
+        log.msg("Registered entity %s" % entity)
+
     def destroy_entity(self, entity):
-        self.entities.discard(entity)
+        """
+        Destroy an entity.
+
+        The factory doesn't have to know about entities, but it is a good
+        place to put this logic.
+        """
+
+        bigx = entity.location.x // 16
+        bigz = entity.location.z // 16
+
+        chunk = self.world.load_chunk(bigx, bigz)
+        chunk.entities.discard(entity)
+
+        log.msg("Destroyed entity %s" % entity)
 
     def update_time(self):
         """
@@ -139,7 +203,9 @@ class BravoFactory(Factory):
         Update the world's season.
         """
 
-        for plugin in retrieve_plugins(ISeason).itervalues():
+        plugins = configuration.getlistdefault(self.config_name,
+            "seasons", [])
+        for plugin in retrieve_named_plugins(ISeason, plugins):
             if plugin.day == self.day:
                 self.world.season = plugin
 
@@ -163,9 +229,25 @@ class BravoFactory(Factory):
         self.broadcast(packet)
 
     def broadcast(self, packet):
+        """
+        Broadcast a packet to all connected players.
+        """
+
         for player in self.protocols.itervalues():
             player.transport.write(packet)
 
+    def broadcast_for_others(self, packet, protocol):
+        """
+        Broadcast a packet to all players except the originating player.
+
+        Useful for certain packets like player entity spawns which should
+        never be reflexive.
+        """
+
+        for player in self.protocols.itervalues():
+            if player is not protocol:
+                player.transport.write(packet)
+
     def broadcast_for_chunk(self, packet, x, z):
         """
         Broadcast a packet to all players that have a certain chunk loaded.
@@ -189,27 +271,14 @@ class BravoFactory(Factory):
                     player.transport.write(packet)
             chunk.clear_damage()
 
-    def entities_near(self, x, y, z, radius):
-        """
-        Given a coordinate and a radius, return all entities within that
-        radius of those coordinates.
-
-        All arguments should be in pixels, not blocks.
-        """
-
-        return [entity for entity in self.entities
-            if sqrt(
-                (entity.location.x - x)**2 +
-                (entity.location.y - y)**2 +
-                (entity.location.z - z)**2
-            ) < radius]
-
     def give(self, coords, block, quantity):
         """
         Spawn a pickup at the specified coordinates.
 
         The coordinates need to be in pixels, not blocks.
 
+        If the size of the stack is too big, multiple stacks will be dropped.
+
         :param tuple coords: coordinates, in pixels
         :param tuple block: key of block or item to drop
         :param int quantity: number of blocks to drop in the stack
@@ -217,17 +286,36 @@ class BravoFactory(Factory):
 
         x, y, z = coords
 
-        entity = self.create_entity(x, y, z, "Pickup")
-        entity.block = block
-        entity.quantity = quantity
+        while quantity > 0:
+            entity = self.create_entity(x // 32, y // 32, z // 32, "Item",
+                item=block, quantity=min(quantity, 64))
 
-        packet = make_packet("pickup", eid=entity.eid, primary=block[0],
-            secondary=block[1], count=quantity, x=x, y=y, z=z, yaw=0, pitch=0,
-            roll=0)
-        self.broadcast(packet)
+            packet = entity.save_to_packet()
+            packet += make_packet("create", eid=entity.eid)
+            self.broadcast(packet)
 
-        packet = make_packet("create", eid=entity.eid)
-        self.broadcast(packet)
+            quantity -= 64
+
+    def stopFactory(self):
+        """
+        Called before factory stops listening on ports. Used to perform
+        shutdown tasks.
+        """
+
+        if not self.world.saving:
+            return
+
+        log.msg("Shutting down; flushing world data...")
+
+        # Flush all dirty chunks to disk.
+        for chunk in self.world.dirty_chunk_cache.itervalues():
+            self.world.save_chunk(chunk)
+
+        # Write back current world time.
+        self.world.time = self.time
+        self.world.serializer.save_level(self.world)
+
+        log.msg("World data saved!")
 
     def pauseProducing(self):
         pass
