@@ -1,11 +1,12 @@
+from collections import namedtuple
 from itertools import chain
 
 from construct import Container, ListContainer
 
+from bravo import blocks
 from bravo.ibravo import IRecipe
 from bravo.packets import make_packet
 from bravo.plugin import retrieve_plugins
-from bravo.serialize import InventorySerializer
 
 def grouper(n, iterable):
     args = [iter(iterable)] * n
@@ -26,14 +27,60 @@ def pad_to_stride(recipe, rstride, cstride):
 
     pad = (None,) * (cstride - rstride)
     g = grouper(rstride, recipe)
-    padded = list(next(g))
+    padded = list(g.next())
     for row in g:
         padded.extend(pad)
         padded.extend(row)
 
     return padded
 
-class Inventory(InventorySerializer):
+class Slot(namedtuple("Slot", "primary, secondary, quantity")):
+    """
+    A slot in an inventory.
+    """
+
+    def holds(self, other):
+        """
+        Whether these slots hold the same item.
+
+        This method is comfortable with other ``Slot`` instances, and also
+        with regular {2,3}-tuples.
+        """
+
+        return self.primary == other[0] and self.secondary == other[1]
+
+    def decrement(self, quantity=1):
+        """
+        Return a copy of this slot, with quantity decremented, or None if the
+        slot is empty.
+        """
+
+        if quantity >= self.quantity:
+            return None
+
+        return self._replace(quantity=self.quantity - quantity)
+
+    def increment(self, quantity=1):
+        """
+        Return a copy of this slot, with quantity incremented.
+
+        For parity with ``decrement()``.
+        """
+
+        return self._replace(quantity=self.quantity + quantity)
+
+    def replace(self, **kwargs):
+        """
+        Exposed version of ``_replace()`` with slot semantics.
+        """
+
+        new = self._replace(**kwargs)
+        if new.quantity == 0:
+            return None
+
+        return new
+
+class Inventory(object):
     """
     Item manager.
 
@@ -132,7 +179,7 @@ class Inventory(InventorySerializer):
             if item.id < 0:
                 items[i] = None
             else:
-                items[i] = item.id, item.damage, item.count
+                items[i] = Slot(item.id, item.damage, item.count)
 
         self.load_from_list(items)
 
@@ -143,8 +190,8 @@ class Inventory(InventorySerializer):
             if item is None:
                 lc.append(Container(primary=-1))
             else:
-                lc.append(Container(primary=item[0], secondary=item[1],
-                    count=item[2]))
+                lc.append(Container(primary=item.primary,
+                    secondary=item.secondary, count=item.quantity))
 
         packet = make_packet("inventory", name=self.identifier,
             length=len(lc), items=lc)
@@ -162,51 +209,126 @@ class Inventory(InventorySerializer):
         # Try to put it in holdables first.
         for stash in (self.holdables, self.storage):
             # Check in two separate loops, to avoid bad grouping patterns.
-            for i, t in enumerate(stash):
-                if t is not None:
-                    primary, secondary, count = t
-
-                    if (primary, secondary) == item and count < 64:
-                        count += quantity
+            for i, slot in enumerate(stash):
+                if slot is not None:
+                    if slot.holds(item) and slot.quantity < 64:
+                        count = slot.quantity + quantity
                         if count > 64:
                             count, quantity = 64, count - 64
                         else:
                             quantity = 0
-                        stash[i] = primary, secondary, count
+                        stash[i] = slot.replace(quantity=count)
                         if not quantity:
                             return True
-            for i, t in enumerate(stash):
-                if t is None:
-                    stash[i] = item[0], item[1], quantity
+            for i, slot in enumerate(stash):
+                if slot is None:
+                    stash[i] = Slot(item[0], item[1], quantity)
                     return True
 
         return False
 
-    def consume(self, item):
+    def consume(self, item, index):
         """
         Attempt to remove a used holdable from the inventory.
 
         A return value of ``False`` indicates that there were no holdables of
-        the given type to consume.
+        the given type and slot to consume.
 
         :param tuple item: a key representing the type of the item
+        :param int slot: which slot was selected
         :returns: whether the item was successfully removed
         """
 
-        for i, t in enumerate(self.holdables):
-            if t is not None:
-                primary, secondary, count = t
+        slot = self.holdables[index]
 
-                if (primary, secondary) == item and count:
-                    count -= 1
-                    if count:
-                        self.holdables[i] = primary, secondary, count
-                    else:
-                        self.holdables[i] = None
-                    return True
+        # Can't really remove things from an empty slot...
+        if slot is None:
+            return False
+
+        if slot.holds(item):
+            self.holdables[index] = slot.decrement()
+            return True
 
         return False
 
+
+    def select_armor(self, index, alternate):
+        """
+        Handle a slot selection on an armor slot.
+        """
+
+        # Special case for armor slots.
+        allowed_items_per_slot = {
+            0: blocks.armor_helmets, 1: blocks.armor_chestplates,
+            2: blocks.armor_leggings, 3: blocks.armor_boots
+        }
+
+        allowed_items = allowed_items_per_slot[index]
+
+        if self.selected is not None:
+            sslot = self.selected
+            if sslot.primary not in allowed_items:
+                return False
+
+            if self.armor[index] is None:
+                # Put one armor piece into the slot, decrement the amount
+                # in the selection.
+                self.armor[index] = sslot.replace(quantity=1)
+                self.selected = sslot.decrement()
+            else:
+                # If both slot and selection are the same item, do nothing.
+                # If not, the quantity needs to be 1, because only one item
+                # fits into the slot, and exchanging slot and selection is not
+                # possible otherwise.
+                if not self.armor[index].holds(sslot) and sslot.quantity == 1:
+                    self.selected, self.armor[index] = self.armor[index], self.selected
+                else:
+                    return False
+        else:
+            if self.armor[index] is None:
+                # Slot and selection are empty, do nothing.
+                return False
+            else:
+                # Move item in the slot into the selection.
+                self.selected = self.armor[index]
+                self.armor[index] = None
+
+        # Yeah, okay, success.
+        return True
+
+    def select_crafted(self, index, alternate):
+        """
+        Handle a slot selection on a crafted output.
+        """
+
+        if self.recipe and self.crafted[0]:
+            if self.selected is None:
+                self.selected = self.crafted[0]
+                self.crafted[0] = None
+            else:
+                sslot = self.selected
+                if sslot.holds(self.recipe.provides[0]):
+                    self.selected = sslot.increment(
+                        self.recipe.provides[1])
+                else:
+                    # Mismatch; don't allow it.
+                    return False
+
+            self.reduce_recipe()
+            self.check_recipes()
+            if self.recipe is None:
+                self.crafted[0] = None
+            else:
+                provides = self.recipe.provides
+                self.crafted[0] = Slot(provides[0][0], provides[0][1],
+                    provides[1])
+
+            return True
+        else:
+            # Forbid placing things in the crafted slot.
+            return False
+
+
     def select(self, slot, alternate=False):
         """
         Handle a slot selection.
@@ -220,74 +342,58 @@ class Inventory(InventorySerializer):
                                was done with a right-click
         """
 
-        l, index = self.container_for_slot(slot)
+        # Look up the container and offset.
+        # If, for any reason, our slot is out-of-bounds, then
+        # container_for_slot will return None. In that case, catch the error
+        # and return False.
+        try:
+            l, index = self.container_for_slot(slot)
+        except TypeError:
+            return False
 
-        if l is self.crafted:
-            # Special case for crafted output.
-            if self.recipe and self.crafted[0]:
-                if self.selected is None:
-                    self.selected = self.crafted[0]
-                    self.crafted[0] = None
-                else:
-                    sprimary, ssecondary, scount = self.selected
-                    if (sprimary, ssecondary) == self.recipe.provides[0]:
-                        scount += self.recipe.provides[1]
-                        self.selected = sprimary, ssecondary, scount
+        if l is self.armor:
+            return self.select_armor(index, alternate)
+        elif l is self.crafted:
+            return self.select_crafted(index, alternate)
+        elif self.selected is not None and l[index] is not None:
+            sslot = self.selected
+            islot = l[index]
+            if islot.holds(sslot):
+                # both contain the same item
+                if sslot.quantity + islot.quantity <= 64:
+                    # Sum of items fits in one slot, so this is easy.
+                    if alternate:
+                        l[index] = islot.increment()
+                        self.selected = sslot.decrement()
                     else:
-                        # Mismatch; don't allow it.
-                        return False
-
-                self.reduce_recipe()
-                self.check_recipes()
-                if self.recipe is None:
-                    self.crafted[0] = None
-                else:
-                    provides = self.recipe.provides
-                    self.crafted[0] = provides[0][0], provides[0][1], provides[1]
-
-                return True
-            else:
-                # Forbid placing things in the crafted slot.
-                return False
-
-        if self.selected is not None and l[index] is not None:
-            sprimary, ssecondary, scount = self.selected
-            iprimary, isecondary, icount = l[index]
-            if ((sprimary, ssecondary) == (iprimary, isecondary) and
-                scount + icount <= 64):
-                if alternate:
-                    icount += 1
-                    scount -= 1
-                    l[index] = iprimary, isecondary, icount + 1
-                    if scount <= 0:
+                        l[index] = islot.increment(sslot.quantity)
                         self.selected = None
-                    else:
-                        self.selected = sprimary, ssecondary, scount
                 else:
-                    l[index] = iprimary, isecondary, scount + icount
-                    self.selected = None
+                    # fill up slot to 64, move left overs to selection
+                    # valid for left and right mouse click
+                    l[index] = islot.replace(quantity=64)
+                    self.selected = sslot.replace(
+                        quantity=sslot.quantity + islot.quantity - 64)
             else:
-                if not alternate:
-                    self.selected, l[index] = l[index], self.selected
+                # Default case: just swap
+                # valid for left and right mouse click
+                self.selected, l[index] = l[index], self.selected
         else:
             if alternate:
                 if self.selected is not None:
-                    sprimary, ssecondary, scount = self.selected
-                    l[index] = sprimary, ssecondary, 1
-                    if scount <= 1:
-                        self.selected = None
-                    else:
-                        self.selected = sprimary, ssecondary, scount - 1
+                    sslot = self.selected
+                    l[index] = sslot.replace(quantity=1)
+                    self.selected = sslot.decrement()
+                elif l[index] is None:
+                    # Right click on empty inventory slot does nothing
+                    return False
                 else:
                     # Logically, l[index] is not None, but self.selected is.
-                    lprimary, lsecondary, lcount = l[index]
-                    scount = lcount // 2
-                    scount, lcount = lcount - scount, scount
-                    if lcount >= 0:
-                        l[index] = lprimary, lsecondary, lcount
-                    else:
-                        l[index] = None
-                    self.selected = lprimary, lsecondary, scount
+                    islot = l[index]
+                    scount = islot.quantity // 2
+                    scount, lcount = islot.quantity - scount, scount
+                    l[index] = islot.replace(quantity=lcount)
+                    self.selected = islot.replace(quantity=scount)
             else:
                 # Default case: just swap.
                 self.selected, l[index] = l[index], self.selected
@@ -301,7 +407,8 @@ class Inventory(InventorySerializer):
                 self.crafted[0] = None
             else:
                 provides = self.recipe.provides
-                self.crafted[0] = provides[0][0], provides[0][1], provides[1]
+                self.crafted[0] = Slot(provides[0][0], provides[0][1],
+                    provides[1])
 
         return True
 
@@ -338,10 +445,8 @@ class Inventory(InventorySerializer):
                     if i is None and j is None:
                         matches_needed -= 1
                     elif i is not None and j is not None:
-                        cprimary, csecondary, ccount = j
                         skey, scount = i
-                        if ((cprimary, csecondary) == skey
-                            and ccount >= scount):
+                        if j.holds(skey) and j.quantity >= scount:
                             matches_needed -= 1
 
                     if matches_needed == 0:
@@ -371,13 +476,8 @@ class Inventory(InventorySerializer):
             if slot is not None:
                 index += offset
                 rcount = slot[1]
-                primary, secondary, ccount = self.crafting[index]
-                ccount -= rcount
-                if ccount:
-                    self.crafting[index] = primary, secondary, ccount
-                else:
-                    self.crafting[index] = None
-
+                slot = self.crafting[index]
+                self.crafting[index] = slot.decrement(rcount)
 
 
 class Equipment(Inventory):
@@ -405,8 +505,9 @@ class Furnace(Inventory):
 
 class ChestStorage(Inventory):
 
-    # XXX maybe not right
-    identifier = 3
+    storage = 27
+
+    identifier = 0
 
 def sync_inventories(src, dst):
     """
