@@ -19,11 +19,12 @@ class BravoIRCClient(IRCClient):
         :param str config: configuration key to use for finding settings
         """
 
-        self.factories = dict((factory.name, factory) for factory in worlds)
+        self.factories = worlds
         for factory in self.factories.itervalues():
             factory.chat_consumers.add(self)
 
         self.config = "irc %s" % config
+        self.name = self.config
 
         self.host = configuration.get(self.config, "server")
         self.nickname = configuration.get(self.config, "nick")
