@@ -1,75 +1,10 @@
-from twisted.plugin import IPlugin
 from zope.interface import implements
 
 from bravo.blocks import blocks, items
-from bravo.entity import tile_entities
+from bravo.entity import Chest, Sign
 from bravo.ibravo import IBuildHook
 from bravo.utilities import split_coords
 
-class Torch(object):
-    """
-    Update metadata for torches.
-
-    You almost certainly want to enable this plugin.
-    """
-
-    implements(IPlugin, IBuildHook)
-
-    def build_hook(self, factory, player, builddata):
-        block, metadata, x, y, z, face = builddata
-
-        if block.slot in (blocks["torch"].slot,
-            blocks["redstone-torch"].slot):
-
-            # Update metadata according to face.
-            if face == "-x":
-                builddata = builddata._replace(metadata=0x2)
-            elif face == "+x":
-                builddata = builddata._replace(metadata=0x1)
-            elif face == "-y":
-                # At the moment, you cannot mount torches on the ceiling. :c
-                return False, builddata
-            elif face == "+y":
-                builddata = builddata._replace(metadata=0x5)
-            elif face == "-z":
-                builddata = builddata._replace(metadata=0x4)
-            elif face == "+z":
-                builddata = builddata._replace(metadata=0x3)
-
-        return True, builddata
-
-    name = "torch"
-
-class Ladder(object):
-    """
-    Update metadata for ladders.
-
-    You almost certainly want to enable this plugin.
-    """
-
-    implements(IPlugin, IBuildHook)
-
-    def build_hook(self, factory, player, builddata):
-        block, metadata, x, y, z, face = builddata
-
-        if block.slot == blocks["ladder"].slot:
-            # Update metadata according to face.
-            if face == "-x":
-                builddata = builddata._replace(metadata=0x4)
-            elif face == "+x":
-                builddata = builddata._replace(metadata=0x5)
-            elif face == "-z":
-                builddata = builddata._replace(metadata=0x2)
-            elif face == "+z":
-                builddata = builddata._replace(metadata=0x3)
-            else:
-                # What would a ceiling ladder even look like?
-                return False, builddata
-
-        return True, builddata
-
-    name = "ladder"
-
 class Tile(object):
     """
     Place tiles.
@@ -77,7 +12,7 @@ class Tile(object):
     You almost certainly want to enable this plugin.
     """
 
-    implements(IPlugin, IBuildHook)
+    implements(IBuildHook)
 
     def build_hook(self, factory, player, builddata):
         item, metadata, x, y, z, face = builddata
@@ -97,8 +32,14 @@ class Tile(object):
                 # Ceiling Sign is watching you read.
                 return False, builddata
             elif face == "+y":
-                # Put +Y signs on signposts. We're fancy that way.
-                builddata = builddata._replace(block=blocks["signpost"])
+                # Put +Y signs on signposts. We're fancy that way. Also,
+                # calculate the proper orientation based on player
+                # orientation.
+                # 180 degrees around to orient the signs correctly, and then
+                # 23 degrees to get the sign to midpoint correctly.
+                metadata = ((player.location.yaw + 180) * 16 // 360) % 0xf
+                builddata = builddata._replace(block=blocks["signpost"],
+                    metadata=metadata)
                 y += 1
             elif face == "-z":
                 builddata = builddata._replace(metadata=0x2)
@@ -111,21 +52,37 @@ class Tile(object):
             chunk = factory.world.load_chunk(bigx, bigz)
 
             # Let's build a sign!
-            s = tile_entities["Sign"]()
-            s.x = x
-            s.y = y
-            s.z = z
+            s = Sign(smallx, y, smallz)
+            chunk.tiles[smallx, y, smallz] = s
 
-            chunk.tiles[x, y, z] = s
+        elif item.slot == blocks["chest"].slot:
+            if face == "-x":
+                x -= 1
+            elif face == "+x":
+                x += 1
+            elif face == "-y":
+                y -= 1
+            elif face == "+y":
+                y += 1
+            elif face == "-z":
+                z -= 1
+            elif face == "+z":
+                z += 1
 
-            # We handled this build correctly, but the signpost still needs to
-            # get placed.
-            return True, builddata
+            bigx, smallx, bigz, smallz = split_coords(x, z)
+            chunk = factory.world.load_chunk(bigx, bigz)
+
+            # Not much to do, just tell the chunk about this chest.
+            c = Chest(smallx, y, smallz)
+            chunk.tiles[smallx, y, smallz] = c
 
         return True, builddata
 
     name = "tile"
 
+    before = tuple()
+    after = ("build",)
+
 class Build(object):
     """
     Place a block in a given location.
@@ -133,7 +90,7 @@ class Build(object):
     You almost certainly want to enable this plugin.
     """
 
-    implements(IPlugin, IBuildHook)
+    implements(IBuildHook)
 
     def build_hook(self, factory, player, builddata):
         block, metadata, x, y, z, face = builddata
@@ -142,9 +99,19 @@ class Build(object):
         if block.slot not in blocks:
             return True, builddata
 
+        # Check for orientable blocks.
+        if not metadata and block.orientable():
+            metadata = block.orientation(face)
+            if metadata is None:
+                # Oh, I guess we can't even place the block on this face.
+                return True, builddata
+
         # Make sure we can remove it from the inventory first.
-        if not player.inventory.consume((block.slot, 0)):
-            return True, builddata
+        if not player.inventory.consume((block.slot, 0), player.equipped):
+            # Okay, first one was a bust; maybe we can consume the related
+            # block for dropping instead?
+            if not player.inventory.consume((block.drop, 0), player.equipped):
+                return True, builddata
 
         # Offset coords according to face.
         if face == "-x":
@@ -160,17 +127,19 @@ class Build(object):
         elif face == "+z":
             z += 1
 
-        bigx, smallx, bigz, smallz = split_coords(x, z)
-        chunk = factory.world.load_chunk(bigx, bigz)
+        builddata = builddata._replace(x=x, y=y, z=z, face="noop")
 
-        chunk.set_block((smallx, y, smallz), block.slot)
+        factory.world.set_block((x, y, z), block.slot)
         if metadata:
-            chunk.set_metadata((smallx, y, smallz), metadata)
+            factory.world.set_metadata((x, y, z), metadata)
 
         return True, builddata
 
     name = "build"
 
+    before = tuple()
+    after = tuple()
+
 class BuildSnow(object):
     """
     Makes building on snow behave correctly.
@@ -178,7 +147,7 @@ class BuildSnow(object):
     You almost certainly want to enable this plugin.
     """
 
-    implements(IPlugin, IBuildHook)
+    implements(IBuildHook)
 
     def build_hook(self, factory, player, builddata):
         bigx, smallx, bigz, smallz = split_coords(builddata.x, builddata.z)
@@ -193,8 +162,9 @@ class BuildSnow(object):
 
     name = "build_snow"
 
-torch = Torch()
+    before = tuple()
+    after = ("build",)
+
 tile = Tile()
 build = Build()
 build_snow = BuildSnow()
-ladder = Ladder()
