@@ -1,11 +1,10 @@
-from twisted.plugin import IPlugin
 from zope.interface import implements
 
 from bravo.ibravo import IConsoleCommand
 
 class Meliae(object):
 
-    implements(IPlugin, IConsoleCommand)
+    implements(IConsoleCommand)
 
     def console_command(self, factory, parameters):
         out = "".join(parameters)
@@ -26,7 +25,7 @@ class Meliae(object):
 
 class Status(object):
 
-    implements(IPlugin, IConsoleCommand)
+    implements(IConsoleCommand)
 
     def console_command(self, factory, parameters):
         protocol_count = len(factory.protocols)
@@ -47,20 +46,5 @@ class Status(object):
     usage = ""
     info = "Print a quick summary of the server's status"
 
-class PDB(object):
-
-    implements(IPlugin, IConsoleCommand)
-
-    def console_command(self, factory, parameters):
-        import pdb; pdb.set_trace()
-
-        return tuple()
-
-    name = "pdb"
-    aliases = tuple()
-    usage = ""
-    info = "Drop into a PDB shell"
-
 meliae = Meliae()
 status = Status()
-pdb = PDB()
