@@ -1,5 +1,8 @@
+from collections import namedtuple
+from itertools import product
 from time import time
 from urlparse import urlunparse
+from math import pi
 
 from twisted.internet import reactor
 from twisted.internet.defer import succeed
@@ -11,27 +14,26 @@ from twisted.python import log
 from twisted.web.client import getPage
 
 from bravo.blocks import blocks, items
-from bravo.compat import namedtuple, product
 from bravo.config import configuration
 from bravo.entity import Sign
 from bravo.factories.infini import InfiniClientFactory
-from bravo.ibravo import IChatCommand, IBuildHook, IDigHook
+from bravo.ibravo import IChatCommand, IBuildHook, IDigHook, ISignHook
 from bravo.inventory import Workbench, sync_inventories
+from bravo.location import Location
 from bravo.packets import parse_packets, make_packet, make_error_packet
-from bravo.plugin import retrieve_plugins, retrieve_named_plugins
+from bravo.plugin import retrieve_plugins, retrieve_sorted_plugins
 from bravo.utilities import split_coords
 
 (STATE_UNAUTHENTICATED, STATE_CHALLENGED, STATE_AUTHENTICATED) = range(3)
 
+SUPPORTED_PROTOCOL = 10
+
 circle = [(i, j)
-    for i, j in sorted(
-        product(xrange(-10, 10), xrange(-10, 10)),
-        key=lambda t: t[0]**2 + t[1]**2)
+    for i, j in product(xrange(-10, 10), xrange(-10, 10))
     if i**2 + j**2 <= 100
 ]
 """
-A list of points in a filled circle of radius 10, sorted according to distance
-from the center.
+A list of points in a filled circle of radius 10.
 """
 
 BuildData = namedtuple("BuildData", "block, metadata, x, y, z, face")
@@ -57,26 +59,26 @@ class BetaServerProtocol(Protocol):
     username = None
 
     def __init__(self):
-        log.msg("Client connecting...")
-
         self.chunks = dict()
-        self.entities = set()
         self.windows = dict()
         self.wid = 1
 
+        self.location = Location()
+
         self.handlers = {
             0: self.ping,
             1: self.login,
             2: self.handshake,
             3: self.chat,
-            10: self.flying,
-            11: self.position_look,
-            12: self.position_look,
-            13: self.position_look,
+            10: self.grounded,
+            11: self.position,
+            12: self.orientation,
+            13: self.location_packet,
             14: self.digging,
             15: self.build,
             16: self.equip,
             18: self.animate,
+            19: self.action,
             21: self.pickup,
             101: self.wclose,
             102: self.waction,
@@ -85,6 +87,12 @@ class BetaServerProtocol(Protocol):
             255: self.quit,
         }
 
+        self._ping_loop = LoopingCall(self.update_ping)
+
+    # Low-level packet handlers
+    # Try not to hook these if possible, since they offer no convenient
+    # abstractions or protections.
+
     def ping(self, container):
         """
         Hook for ping packets.
@@ -100,14 +108,14 @@ class BetaServerProtocol(Protocol):
         callback.
         """
 
-        if container.protocol < 8:
+        if container.protocol < SUPPORTED_PROTOCOL:
             # Kick old clients.
             self.error("This server doesn't support your ancient client.")
-        elif container.protocol > 8:
+        elif container.protocol > SUPPORTED_PROTOCOL:
             # Kick new clients.
             self.error("This server doesn't support your newfangled client.")
-
-        reactor.callLater(0, self.authenticated)
+        else:
+            reactor.callLater(0, self.authenticated)
 
     def handshake(self, container):
         """
@@ -119,17 +127,59 @@ class BetaServerProtocol(Protocol):
         Hook for chat packets.
         """
 
-    def flying(self, container):
+    def grounded(self, container):
         """
-        Hook for flying packets.
+        Hook for grounded packets.
         """
 
-    def position_look(self, container):
-        """
-        Hook for position and look packets.
+        self.location.grounded = bool(container.grounded)
 
-        XXX this design decision should be revisted
+    def position(self, container):
         """
+        Hook for position packets.
+        """
+
+        old_position = self.location.x, self.location.y, self.location.z
+
+        self.location.x = int(container.position.x)
+        self.location.y = int(container.position.y)
+        self.location.z = int(container.position.z)
+        # Stance is the current jumping position, plus a small offset of
+        # around 0.1. In the Alpha server, it must between 0.1 and 1.65,
+        # or the anti-grounded code kicks the client.
+        self.location.stance = container.position.stance
+
+        position = self.location.x, self.location.y, self.location.z
+
+        self.grounded(container.grounded)
+
+        if old_position != position:
+            self.position_changed()
+
+    def orientation(self, container):
+        """
+        Hook for orientation packets.
+        """
+
+        old_orientation = self.location.yaw, self.location.pitch
+
+        self.location.yaw = container.orientation.rotation
+        self.location.pitch = container.orientation.pitch
+
+        orientation = self.location.yaw, self.location.pitch
+
+        self.grounded(container.grounded)
+
+        if old_orientation != orientation:
+            self.orientation_changed()
+
+    def location_packet(self, container):
+        """
+        Hook for location packets.
+        """
+
+        self.position(container)
+        self.orientation(container)
 
     def digging(self, container):
         """
@@ -156,6 +206,11 @@ class BetaServerProtocol(Protocol):
         Hook for animate packets.
         """
 
+    def action(self, container):
+        """
+        Hook for action packets.
+        """
+
     def wclose(self, container):
         """
         Hook for wclose packets.
@@ -177,9 +232,23 @@ class BetaServerProtocol(Protocol):
         """
 
     def quit(self, container):
+        """
+        Hook for quit packets.
+
+        By default, merely logs the quit message and drops the connection.
+
+        Even if the connection is not dropped, it will be lost anyway since
+        the client will close the connection. It's better to explicitly let it
+        go here than to have zombie protocols.
+        """
+
         log.msg("Client is quitting: %s" % container.message)
         self.transport.loseConnection()
 
+    # Twisted-level data handlers and methods
+    # Please don't override these needlessly, as they are pretty solid and
+    # shouldn't need to be touched.
+
     def dataReceived(self, data):
         self.buf += data
 
@@ -192,7 +261,18 @@ class BetaServerProtocol(Protocol):
                 log.err("Didn't handle parseable packet %d!" % header)
                 log.err(payload)
 
+    def connectionLost(self, reason):
+        if self._ping_loop.running:
+            self._ping_loop.stop()
+
+    # State-change callbacks
+    # Feel free to override these, but call them at some point.
+
     def challenged(self):
+        """
+        Called when the client has started authentication with the server.
+        """
+
         self.state = STATE_CHALLENGED
 
     def authenticated(self):
@@ -202,10 +282,63 @@ class BetaServerProtocol(Protocol):
 
         self.state = STATE_AUTHENTICATED
 
+        self._ping_loop.start(5)
+
+    # Event callbacks
+    # These are meant to be overriden.
+
+    def orientation_changed(self):
+        """
+        Called when the client moves.
+
+        This callback is only for orientation, not position.
+        """
+
+        pass
+
+    def position_changed(self):
+        """
+        Called when the client moves.
+
+        This callback is only for position, not orientation.
+        """
+
+        pass
+
+    # Convenience methods
+
+    def update_ping(self):
+        """
+        Send a keepalive to the client.
+        """
+
+        packet = make_packet("ping")
+        self.transport.write(packet)
+
     def error(self, message):
+        """
+        Error out.
+
+        This method sends ``message`` to the client as a descriptive error
+        message, then closes the connection.
+        """
+
         self.transport.write(make_error_packet(message))
         self.transport.loseConnection()
 
+
+class BannedProtocol(BetaServerProtocol):
+    """
+    A very simple Beta protocol that helps enforce IP bans.
+
+    This protocol disconnects people as soon as they connect, with a helpful
+    message.
+    """
+
+    def connectionMade(self):
+        self.error("Sorry, but your IP address is banned.")
+
+
 class BetaProxyProtocol(BetaServerProtocol):
     """
     A ``BetaServerProtocol`` that proxies for an InfiniCraft client.
@@ -256,6 +389,7 @@ class BetaProxyProtocol(BetaServerProtocol):
         log.err("Couldn't connect node!")
         log.err(reason)
 
+
 class BravoProtocol(BetaServerProtocol):
     """
     A ``BetaServerProtocol`` suitable for serving MC worlds to clients.
@@ -267,45 +401,59 @@ class BravoProtocol(BetaServerProtocol):
     chunk_tasks = None
 
     time_loop = None
-    ping_loop = None
 
-    def __init__(self):
+    eid = 0
+
+    def __init__(self, name):
         BetaServerProtocol.__init__(self)
 
+        self.config_name = "world %s" % name
+
         log.msg("Registering client hooks...")
-        names = configuration.getlist("bravo", "build_hooks")
-        self.build_hooks = retrieve_named_plugins(IBuildHook, names)
-        names = configuration.getlist("bravo", "dig_hooks")
-        self.dig_hooks = retrieve_named_plugins(IDigHook, names)
+        names = configuration.getlistdefault(self.config_name, "build_hooks",
+            [])
+        self.build_hooks = retrieve_sorted_plugins(IBuildHook, names)
+        names = configuration.getlistdefault(self.config_name, "dig_hooks",
+            [])
+        self.dig_hooks = retrieve_sorted_plugins(IDigHook, names)
+        names = configuration.getlistdefault(self.config_name, "sign_hooks",
+            [])
+        self.sign_hooks = retrieve_sorted_plugins(ISignHook, names)
 
         self.last_dig_build_timer = time()
 
-    def challenged(self):
-        BetaServerProtocol.challenged(self)
-
-        # Maybe the ugliest hack written thus far.
-        # We need an entity ID which will persist for the entire lifetime of
-        # this client. However, that entity ID is normally tied to an entity,
-        # which won't be allocated until after we get our username from the
-        # client. This is far too late to be able to look things up in a nice,
-        # orderly way, so for now (and maybe forever) we will allocate and
-        # increment the entity ID manually.
-        self.eid = self.factory.eid + 1
-        self.factory.eid += 1
-
     def authenticated(self):
         BetaServerProtocol.authenticated(self)
 
-        self.factory.protocols[self.username] = self
-
+        # Init player, and copy data into it.
         self.player = self.factory.world.load_player(self.username)
         self.player.eid = self.eid
-        self.factory.entities.add(self.player)
+        self.location = self.player.location
 
+        # Announce our presence.
         packet = make_packet("chat",
             message="%s is joining the game..." % self.username)
         self.factory.broadcast(packet)
 
+        # Craft our avatar and send it to already-connected other players.
+        packet = self.player.save_to_packet()
+        packet += make_packet("create", eid=self.player.eid)
+        self.factory.broadcast_for_others(packet, self)
+
+        # And of course spawn all of those players' avatars in our client as
+        # well. Note that, at this point, we are not listed in the factory's
+        # list of protocols, so we won't accidentally send one of these to
+        # ourselves.
+        for protocol in self.factory.protocols.itervalues():
+            packet = protocol.player.save_to_packet()
+            self.transport.write(packet)
+            packet = protocol.player.save_equipment_to_packet()
+            self.transport.write(packet)
+            packet = make_packet("create", eid=protocol.player.eid)
+            self.transport.write(packet)
+
+        self.factory.protocols[self.username] = self
+
         spawn = self.factory.world.spawn
         packet = make_packet("spawn", x=spawn[0], y=spawn[1], z=spawn[2])
         self.transport.write(packet)
@@ -315,12 +463,74 @@ class BravoProtocol(BetaServerProtocol):
 
         self.send_initial_chunk_and_location()
 
-        self.ping_loop = LoopingCall(self.update_ping)
-        self.ping_loop.start(5)
-
         self.time_loop = LoopingCall(self.update_time)
         self.time_loop.start(10)
 
+    def orientation_changed(self):
+        # Bang your head!
+        packet = make_packet("entity-orientation",
+            eid=self.player.eid,
+            yaw=int(self.location.theta * 255 / (2 * pi)) % 256,
+            pitch=int(self.location.phi * 255 / (2 * pi)) % 256,
+        )
+        self.factory.broadcast_for_others(packet, self)
+
+    def position_changed(self):
+        x, chaff, z, chaff = split_coords(self.location.x, self.location.z)
+
+        # Inform everybody of our new location.
+        packet = make_packet("teleport",
+            eid=self.player.eid,
+            x=self.location.x * 32,
+            y=self.location.y * 32,
+            z=self.location.z * 32,
+            yaw=int(self.location.theta * 255 / (2 * pi)) % 256,
+            pitch=int(self.location.phi * 255 / (2 * pi)) % 256,
+        )
+        self.factory.broadcast_for_others(packet, self)
+
+        self.update_chunks()
+
+        for entity in self.entities_near(2):
+            if entity.name != "Item":
+                continue
+
+            if self.player.inventory.add(entity.item, entity.quantity):
+                packet = make_packet("collect", eid=entity.eid,
+                    destination=self.player.eid)
+                self.factory.broadcast(packet)
+
+                packet = make_packet("destroy", eid=entity.eid)
+                self.factory.broadcast(packet)
+
+                packet = self.player.inventory.save_to_packet()
+                self.transport.write(packet)
+
+                self.factory.destroy_entity(entity)
+
+    def entities_near(self, radius):
+        """
+        Obtain the entities within a radius of this player.
+
+        Radius is measured in blocks.
+        """
+
+        chunk_radius = int(radius // 16 + 1)
+        chunkx, chaff, chunkz, chaff = split_coords(self.location.x,
+            self.location.z)
+
+        minx = chunkx - chunk_radius
+        maxx = chunkx + chunk_radius + 1
+        minz = chunkz - chunk_radius
+        maxz = chunkz + chunk_radius + 1
+
+        for x, z in product(xrange(minx, maxx), xrange(minz, maxz)):
+            chunk = self.chunks[x, z]
+            yieldables = [entity for entity in chunk.entities
+                if self.location.distance(entity.location) <= radius]
+            for i in yieldables:
+                yield i
+
     def login(self, container):
         """
         Handle a login packet.
@@ -331,13 +541,25 @@ class BravoProtocol(BetaServerProtocol):
         them depending on the results of the authentication.
         """
 
+        if container.protocol < SUPPORTED_PROTOCOL:
+            # Kick old clients.
+            self.error("This server doesn't support your ancient client.")
+            return
+        elif container.protocol > SUPPORTED_PROTOCOL:
+            # Kick new clients.
+            self.error("This server doesn't support your newfangled client.")
+            return
+
+        log.msg("Authenticating client, protocol version %d" %
+            container.protocol)
+
         d = self.factory.login_hook(self, container)
         d.addErrback(lambda *args, **kwargs: self.transport.loseConnection())
         d.addCallback(lambda *args, **kwargs: self.authenticated())
 
     def handshake(self, container):
         if not self.factory.handshake_hook(self, container):
-            self.loseConnection()
+            self.transport.loseConnection()
 
     def chat(self, container):
         if container.message.startswith("/"):
@@ -372,59 +594,46 @@ class BravoProtocol(BetaServerProtocol):
             message = "<%s> %s" % (self.username, container.message)
             self.factory.chat(message)
 
-    def flying(self, container):
-        self.player.location.load_from_packet(container)
-
-    def position_look(self, container):
-        oldx, chaff, oldz, chaff = split_coords(self.player.location.x,
-            self.player.location.z)
-
-        self.player.location.load_from_packet(container)
-
-        pos = (self.player.location.x, self.player.location.y,
-            self.player.location.z)
-
-        x, chaff, z, chaff = split_coords(pos[0], pos[2])
-
-        if oldx != x or oldz != z:
-            self.update_chunks()
-
-        for entity in self.factory.entities_near(pos[0] * 32,
-            pos[1] * 32, pos[2] * 32, 2 * 32):
-            if entity.name != "Pickup":
-                continue
-
-            if self.player.inventory.add(entity.block, entity.quantity):
-                packet = self.player.inventory.save_to_packet()
-                self.transport.write(packet)
-
-                packet = make_packet("collect", eid=entity.eid,
-                    destination=self.player.eid)
-                self.transport.write(packet)
-
-                packet = make_packet("destroy", eid=entity.eid)
-                self.transport.write(packet)
-
-                self.factory.destroy_entity(entity)
-
-        for entity in self.factory.entities_near(pos[0] * 32,
-            pos[1] * 32, pos[2] * 32, 160 * 32):
-
-            if (entity is self.player or
-                entity.name != "Player" or
-                entity.eid in self.entities):
-                continue
-
-            self.entities.add(entity.eid)
-
-            packet = entity.save_to_packet()
-            self.transport.write(packet)
-
-            packet = make_packet("create", eid=entity.eid)
-            self.transport.write(packet)
-
     def digging(self, container):
-        if container.state != 3:
+        # XXX several improvements should happen here
+        # ~ We should time started and stopped pairs to force clients to
+        # slow-break their blocks
+        if container.x == -1 and container.z == -1 and container.y == 255:
+            # Lala-land dig packet. Discard it for now.
+            return
+
+        # Player drops currently holding item/block.
+        if (container.state == "dropped" and container.face == "-y" and
+            container.x == 0 and container.y == 0 and container.z == 0):
+            i = self.player.inventory
+            holding = i.holdables[self.player.equipped]
+            if holding:
+                primary, secondary, count = holding
+                if i.consume((primary, secondary), self.player.equipped):
+                    dest = self.location.in_front_of(2)
+                    dest.y += 1
+                    coords = (int(dest.x * 32) + 16, int(dest.y * 32) + 16,
+                        int(dest.z * 32) + 16)
+                    self.factory.give(coords, (primary, secondary), 1)
+
+                    # Re-send inventory.
+                    packet = self.player.inventory.save_to_packet()
+                    self.transport.write(packet)
+
+                    # If no items in this slot are left, this player isn't
+                    # holding an item anymore.
+                    if i.holdables[self.player.equipped] is None:
+                        packet = make_packet("entity-equipment",
+                            eid=self.player.eid,
+                            slot=0,
+                            primary=65535,
+                            secondary=0
+                        )
+                        self.factory.broadcast_for_others(packet, self)
+            return
+
+        if container.state != "stopped":
+            # We only care about digs which break blocks.
             return
 
         if time() - self.last_dig_build_timer < 0.1:
@@ -449,6 +658,10 @@ class BravoProtocol(BetaServerProtocol):
         self.factory.flush_chunk(chunk)
 
     def build(self, container):
+        if container.x == -1 and container.z == -1 and container.y == 255:
+            # Lala-land build packet. Discard it for now.
+            return
+
         # Is the target being selected?
         bigx, smallx, bigz, smallz = split_coords(container.x, container.z)
         try:
@@ -486,11 +699,15 @@ class BravoProtocol(BetaServerProtocol):
                 container.primary)
             return
 
-        if time() - self.last_dig_build_timer < 0.1:
+        if time() - self.last_dig_build_timer < 0.05:
             self.error("You are building too fast.")
 
         self.last_dig_build_timer = time()
 
+        # it's the top of the world, you can't build here
+        if container.y == 127 and container.face == '+y':
+            return
+
         builddata = BuildData(block, 0x0, container.x, container.y,
             container.z, container.face)
 
@@ -512,13 +729,49 @@ class BravoProtocol(BetaServerProtocol):
     def equip(self, container):
         self.player.equipped = container.item
 
+        # Inform everyone about the item the player is holding now.
+        item = self.player.inventory.holdables[self.player.equipped]
+        if item is None:
+            # Empty slot. Use signed short -1 == unsigned 65535.
+            primary, secondary = 65535, 0
+        else:
+            primary, secondary, count = item
+
+        packet = make_packet("entity-equipment",
+            eid=self.player.eid,
+            slot=0,
+            primary=primary,
+            secondary=secondary
+        )
+        self.factory.broadcast_for_others(packet, self)
+
     def pickup(self, container):
         self.factory.give((container.x, container.y, container.z),
             (container.primary, container.secondary), container.count)
 
+    def animate(self, container):
+        # Broadcast the animation of the entity to everyone else. Only swing
+        # arm is send by notchian clients.
+        packet = make_packet("animate",
+            eid=self.player.eid,
+            animation=container.animation
+        )
+        self.factory.broadcast_for_others(packet, self)
+
     def wclose(self, container):
         if container.wid in self.windows:
             i = self.windows[container.wid]
+            if i.identifier == 1:
+                # Closing the workbench.
+                dest = self.location.in_front_of(1)
+                dest.y += 1
+                coords = (int(dest.x * 32) + 16, int(dest.y * 32) + 16,
+                    int(dest.z * 32) + 16)
+                # loop over items left in workbench
+                for item in i.crafting:
+                    if item is None:
+                        continue
+                    self.factory.give(coords, (item[0], item[1]), item[2])
             del self.windows[container.wid]
             sync_inventories(i, self.player.inventory)
         elif container.wid == 0:
@@ -542,6 +795,32 @@ class BravoProtocol(BetaServerProtocol):
             packet = i.save_to_packet()
             self.transport.write(packet)
 
+            # Inform other players about changes to this player's equipment.
+            if container.wid == 0 and (container.slot in range(5, 9) or
+                                       container.slot == 36):
+
+                # Armor changes.
+                if container.slot in range(5, 9):
+                    item = i.armor[container.slot - 5]
+                    # Order of slots is reversed in the equipment package.
+                    slot = 4 - (container.slot - 5)
+                # Currently equipped item changes.
+                elif container.slot == 36:
+                    item = i.holdables[0]
+                    slot = 0
+
+                if item is None:
+                    primary, secondary = 65535, 0
+                else:
+                    primary, secondary, count = item
+                packet = make_packet("entity-equipment",
+                    eid=self.player.eid,
+                    slot=slot,
+                    primary=primary,
+                    secondary=secondary
+                )
+                self.factory.broadcast_for_others(packet, self)
+
         packet = make_packet("window-token", wid=0, token=container.token,
             acknowledged=selected)
         self.transport.write(packet)
@@ -556,20 +835,18 @@ class BravoProtocol(BetaServerProtocol):
             return
 
         if (smallx, container.y, smallz) in chunk.tiles:
+            new = False
             s = chunk.tiles[smallx, container.y, smallz]
         else:
-            s = Sign()
-
-        s.x = smallx
-        s.y = container.y
-        s.z = smallz
+            new = True
+            s = Sign(smallx, container.y, smallz)
+            chunk.tiles[smallx, container.y, smallz] = s
 
         s.text1 = container.line1
         s.text2 = container.line2
         s.text3 = container.line3
         s.text4 = container.line4
 
-        chunk.tiles[smallx, container.y, smallz] = s
         chunk.dirty = True
 
         # The best part of a sign isn't making one, it's showing everybody
@@ -577,8 +854,18 @@ class BravoProtocol(BetaServerProtocol):
         packet = make_packet("sign", container)
         self.factory.broadcast_for_chunk(packet, bigx, bigz)
 
+        # Run sign hooks.
+        for hook in self.sign_hooks:
+            hook.sign_hook(self.factory, chunk, container.x, container.y,
+                container.z, [s.text1, s.text2, s.text3, s.text4], new)
+
     def disable_chunk(self, x, z):
-        del self.chunks[x, z]
+        # Remove the chunk from cache.
+        chunk = self.chunks.pop(x, z)
+
+        for entity in chunk.entities:
+            packet = make_packet("destroy", eid=entity.eid)
+            self.transport.write(packet)
 
         packet = make_packet("prechunk", x=x, z=z, enabled=0)
         self.transport.write(packet)
@@ -609,15 +896,23 @@ class BravoProtocol(BetaServerProtocol):
         packet = chunk.save_to_packet()
         self.transport.write(packet)
 
-        for entity in chunk.tiles.itervalues():
+        for entity in chunk.entities:
             packet = entity.save_to_packet()
             self.transport.write(packet)
 
+            packet = make_packet("create", eid=entity.eid)
+            self.transport.write(packet)
+
+        for entity in chunk.tiles.itervalues():
+            if entity.name == "Sign":
+                packet = entity.save_to_packet()
+                self.transport.write(packet)
+
         self.chunks[chunk.x, chunk.z] = chunk
 
     def send_initial_chunk_and_location(self):
-        bigx, smallx, bigz, smallz = split_coords(self.player.location.x,
-            self.player.location.z)
+        bigx, smallx, bigz, smallz = split_coords(self.location.x,
+            self.location.z)
 
         # Spawn the 25 chunks in a square around the spawn, *before* spawning
         # the player. Otherwise, there's a funky Beta 1.2 bug which causes the
@@ -633,23 +928,23 @@ class BravoProtocol(BetaServerProtocol):
         # Don't dare send more chunks beyond the initial one until we've
         # spawned.
         d.addCallback(lambda none: self.update_location())
+        d.addCallback(lambda none: self.position_changed())
         d.addCallback(lambda none: self.update_chunks())
 
     def update_location(self):
-        bigx, smallx, bigz, smallz = split_coords(self.player.location.x,
-            self.player.location.z)
+        bigx, smallx, bigz, smallz = split_coords(self.location.x,
+            self.location.z)
 
         chunk = self.chunks[bigx, bigz]
 
         height = chunk.height_at(smallx, smallz) + 2
-        self.player.location.y = height
+        self.location.y = height
 
-        packet = self.player.location.save_to_packet()
+        packet = self.location.save_to_packet()
         self.transport.write(packet)
 
     def update_chunks(self):
-        x, chaff, z, chaff = split_coords(self.player.location.x,
-            self.player.location.z)
+        x, chaff, z, chaff = split_coords(self.location.x, self.location.z)
 
         new = set((i + x, j + z) for i, j in circle)
         old = set(self.chunks.iterkeys())
@@ -678,17 +973,11 @@ class BravoProtocol(BetaServerProtocol):
             (self.disable_chunk(i, j) for i, j in discarded)
         ]
 
-    def update_ping(self):
-        packet = make_packet("ping")
-        self.transport.write(packet)
-
     def update_time(self):
         packet = make_packet("time", timestamp=int(self.factory.time))
         self.transport.write(packet)
 
     def connectionLost(self, reason):
-        if self.ping_loop:
-            self.ping_loop.stop()
         if self.time_loop:
             self.time_loop.stop()
 
@@ -699,11 +988,12 @@ class BravoProtocol(BetaServerProtocol):
                 except (TaskDone, TaskFailed):
                     pass
 
-        del self.chunks
-
         if self.player:
             self.factory.world.save_player(self.username, self.player)
             self.factory.destroy_entity(self.player)
+            packet = make_packet("destroy", eid=self.player.eid)
+            self.factory.broadcast(packet)
+            self.factory.chat("%s has left the game." % self.username)
 
         if self.username in self.factory.protocols:
             del self.factory.protocols[self.username]
