@@ -1,3 +1,5 @@
+from functools import wraps
+from itertools import product
 import random
 import sys
 import weakref
@@ -5,63 +7,37 @@ import weakref
 from numpy import fromstring, uint8
 
 from twisted.internet import reactor
-from twisted.internet.defer import Deferred, succeed
+from twisted.internet.defer import succeed
 from twisted.internet.task import coiterate, deferLater, LoopingCall
 from twisted.python import log
-from twisted.python.filepath import FilePath
 
 from bravo.chunk import Chunk
-from bravo.compat import product
 from bravo.config import configuration
-from bravo.serialize import LevelSerializer
-from bravo.serialize import read_from_file, write_to_file, extension
-from bravo.utilities import split_coords
+from bravo.entity import Player
+from bravo.errors import SerializerReadException
+from bravo.ibravo import ISerializer, ISerializerFactory
+from bravo.plugin import (retrieve_named_plugins, verify_plugin,
+    PluginException)
+from bravo.utilities import fork_deferred, split_coords
 
-try:
-    from ampoule import deferToAMPProcess
-    from bravo.remote import MakeChunk
-    async = configuration.getboolean("bravo", "ampoule")
-except ImportError:
-    async = False
-
-def base36(i):
+def coords_to_chunk(f):
     """
-    Return the string representation of i in base 36, using lowercase letters.
+    Automatically look up the chunk for the coordinates, and convert world
+    coordinates to chunk coordinates.
     """
 
-    letters = "0123456789abcdefghijklmnopqrstuvwxyz"
+    @wraps(f)
+    def decorated(self, coords, *args, **kwargs):
+        x, y, z = coords
 
-    if i < 0:
-        i = -i
-        signed = True
-    elif i == 0:
-        return "0"
-    else:
-        signed = False
+        bigx, smallx, bigz, smallz = split_coords(x, z)
+        chunk = self.load_chunk(bigx, bigz)
 
-    s = ""
+        return f(self, chunk, (smallx, y, smallz), *args, **kwargs)
 
-    while i:
-        i, digit = divmod(i, 36)
-        s = letters[digit] + s
+    return decorated
 
-    if signed:
-        s = "-" + s
-
-    return s
-
-def names_for_chunk(x, z):
-    """
-    Calculate the folder and file names for given chunk coordinates.
-    """
-
-    first = base36(x & 63)
-    second = base36(z & 63)
-    third = "c.%s.%s%s" % (base36(x), base36(z), extension())
-
-    return first, second, third
-
-class World(LevelSerializer):
+class World(object):
     """
     Object representing a world on disk.
 
@@ -79,18 +55,37 @@ class World(LevelSerializer):
     Whether objects belonging to this world may be written out to disk.
     """
 
-    def __init__(self, folder):
+    async = False
+    """
+    Whether this world is using multiprocessing methods to generate geometry.
+    """
+
+    dimension = 0
+    """
+    The world dimension.
+
+    XXX Currently pegged to 0; change this when Nether work gets underway.
+    """
+
+    def __init__(self, name):
         """
         Load a world from disk.
 
         :Parameters:
-            folder : str
-                The directory containing the world.
+            name : str
+                The configuration key to use to look up configuration data.
         """
 
-        self.folder = FilePath(folder)
-        if not self.folder.exists():
-            self.folder.makedirs()
+        self.config_name = "world %s" % name
+        world_url = configuration.get(self.config_name, "url")
+        world_sf_name = configuration.get(self.config_name, "serializer")
+
+        try:
+            sf = retrieve_named_plugins(ISerializerFactory, [world_sf_name])[0]
+            self.serializer = verify_plugin(ISerializer, sf(world_url))
+        except PluginException, pe:
+            log.msg(pe)
+            raise RuntimeError("Fatal error: Couldn't set up serializer!")
 
         self.chunk_cache = weakref.WeakValueDictionary()
         self.dirty_chunk_cache = dict()
@@ -99,29 +94,65 @@ class World(LevelSerializer):
 
         self.spawn = (0, 0, 0)
         self.seed = random.randint(0, sys.maxint)
+        self.time = 0
 
-        level = self.folder.child("level%s" % extension())
-        if level.exists() and level.getsize():
-            self.load_from_tag(read_from_file(level.open("r")))
+        # Check if we should offload chunk requests to ampoule.
+        if configuration.getbooleandefault("bravo", "ampoule", False):
+            try:
+                import ampoule
+            except ImportError:
+                pass
+            else:
+                self.async = True
 
-        write_to_file(self.save_to_tag(), level.open("w"))
+
+        # First, try loading the level, to see if there's any data out there
+        # which we can use. If not, don't worry about it.
+        try:
+            self.serializer.load_level(self)
+        except SerializerReadException, sre:
+            log.msg("Had issues loading level data...")
+            log.msg(sre)
+
+        # And now save our level.
+        if self.saving:
+            self.serializer.save_level(self)
 
         self.chunk_management_loop = LoopingCall(self.sort_chunks)
         self.chunk_management_loop.start(1)
 
-    def enable_cache(self):
+        log.msg("World started on %s, using serializer %s" %
+            (world_url, self.serializer.name))
+        log.msg("Using Ampoule: %s" % self.async)
+
+    def enable_cache(self, size):
         """
-        Start up a rudimentary permanent cache.
+        Set the permanent cache size.
+
+        Changing the size of the cache sets off a series of events which will
+        empty or fill the cache to make it the proper size.
+
+        For reference, 3 is a large-enough size to completely satisfy the
+        Notchian client's login demands. 10 is enough to completely fill the
+        Notchian client's chunk buffer.
+
+        :param int size: The taxicab radius of the cache, in chunks
         """
 
+        log.msg("Setting cache size to %d..." % size)
+
         self.permanent_cache = set()
         def assign(chunk):
             self.permanent_cache.add(chunk)
 
-        rx = xrange(self.spawn[0] - 3, self.spawn[0] + 3)
-        rz = xrange(self.spawn[2] - 3, self.spawn[2] + 3)
-        d = coiterate(assign(self.load_chunk(x, z)) for x, z in product(rx, rz))
-        d.addCallback(lambda chaff: log.msg("Cache is warmed up!"))
+        x = self.spawn[0] // 16
+        z = self.spawn[2] // 16
+
+        rx = xrange(x - size, x + size)
+        rz = xrange(z - size, z + size)
+        d = coiterate(self.request_chunk(x, z).addCallback(assign)
+            for x, z in product(rx, rz))
+        d.addCallback(lambda chaff: log.msg("Cache size is now %d" % size))
 
     def sort_chunks(self):
         """
@@ -192,6 +223,28 @@ class World(LevelSerializer):
 
         chunk.regenerate()
 
+    def postprocess_chunk(self, chunk):
+        """
+        Do a series of final steps to bring a chunk into the world.
+        """
+
+        # Apply the current season to the chunk.
+        if self.season:
+            self.season.transform(chunk)
+
+        # Since this chunk hasn't been given to any player yet, there's no
+        # conceivable way that any meaningful damage has been accumulated;
+        # anybody loading any part of this chunk will want the entire thing.
+        # Thus, it should start out undamaged.
+        chunk.clear_damage()
+
+        # Register the chunk's entities with our parent factory.
+        for entity in chunk.entities:
+            self.factory.register_entity(entity)
+
+        # Return the chunk, in case we are in a Deferred chain.
+        return chunk
+
     def request_chunk(self, x, z):
         """
         Request a ``Chunk`` to be delivered later.
@@ -199,36 +252,35 @@ class World(LevelSerializer):
         :returns: Deferred that will be called with the Chunk
         """
 
-        if not async:
-            return deferLater(reactor, 0.000001, self.factory.world.load_chunk,
+        if not self.async:
+            return deferLater(reactor, 0.000001, self.load_chunk,
                 x, z)
 
+        from ampoule import deferToAMPProcess
+        from bravo.remote import MakeChunk
+
         if (x, z) in self.chunk_cache:
             return succeed(self.chunk_cache[x, z])
         elif (x, z) in self.dirty_chunk_cache:
             return succeed(self.dirty_chunk_cache[x, z])
         elif (x, z) in self._pending_chunks:
             # Rig up another Deferred and wrap it up in a to-go box.
-            d = Deferred()
-            self._pending_chunks[x, z].chainDeferred(d)
-            return d
+            return fork_deferred(self._pending_chunks[x, z])
 
         chunk = Chunk(x, z)
-
-        first, second, filename = names_for_chunk(x, z)
-        f = self.folder.child(first).child(second)
-        if not f.exists():
-            f.makedirs()
-        f = f.child(filename)
-        if f.exists() and f.getsize():
-            chunk.load_from_tag(read_from_file(f.open("r")))
+        self.serializer.load_chunk(chunk)
 
         if chunk.populated:
             self.chunk_cache[x, z] = chunk
+            self.postprocess_chunk(chunk)
             return succeed(chunk)
 
-        d = deferToAMPProcess(MakeChunk, x=x, z=z, seed=self.seed,
-            generators=configuration.getlist("bravo", "generators"))
+        d = deferToAMPProcess(MakeChunk,
+            x=x,
+            z=z,
+            seed=self.seed,
+            generators=configuration.getlist(self.config_name, "generators")
+        )
         self._pending_chunks[x, z] = d
 
         def pp(kwargs):
@@ -246,15 +298,7 @@ class World(LevelSerializer):
             chunk.populated = True
             chunk.dirty = True
 
-            # Apply the current season to the chunk.
-            if self.season:
-                self.season.transform(chunk)
-
-            # Since this chunk hasn't been given to any player yet, there's no
-            # conceivable way that any meaningful damage has been accumulated;
-            # anybody loading any part of this chunk will want the entire thing.
-            # Thus, it should start out undamaged.
-            chunk.clear_damage()
+            self.postprocess_chunk(chunk)
 
             self.dirty_chunk_cache[x, z] = chunk
             del self._pending_chunks[x, z]
@@ -263,13 +307,11 @@ class World(LevelSerializer):
 
         # Set up callbacks.
         d.addCallback(pp)
+
         # Multiple people might be subscribed to this pending callback. We're
         # going to keep it for ourselves and fork off another Deferred for our
         # caller.
-        forked = Deferred()
-        d.chainDeferred(forked)
-        forked.addCallback(lambda none: chunk)
-        return forked
+        return fork_deferred(d)
 
     def load_chunk(self, x, z):
         """
@@ -285,14 +327,7 @@ class World(LevelSerializer):
             return self.dirty_chunk_cache[x, z]
 
         chunk = Chunk(x, z)
-
-        first, second, filename = names_for_chunk(x, z)
-        f = self.folder.child(first).child(second)
-        if not f.exists():
-            f.makedirs()
-        f = f.child(filename)
-        if f.exists() and f.getsize():
-            chunk.load_from_tag(read_from_file(f.open("r")))
+        self.serializer.load_chunk(chunk)
 
         if chunk.populated:
             self.chunk_cache[x, z] = chunk
@@ -303,29 +338,16 @@ class World(LevelSerializer):
 
             self.dirty_chunk_cache[x, z] = chunk
 
-        # Apply the current season to the chunk.
-        if self.season:
-            self.season.transform(chunk)
-
-        # Since this chunk hasn't been given to any player yet, there's no
-        # conceivable way that any meaningful damage has been accumulated;
-        # anybody loading any part of this chunk will want the entire thing.
-        # Thus, it should start out undamaged.
-        chunk.clear_damage()
+        self.postprocess_chunk(chunk)
 
         return chunk
 
     def save_chunk(self, chunk):
 
-        if not chunk.dirty:
+        if not chunk.dirty or not self.saving:
             return
 
-        first, second, filename = names_for_chunk(chunk.x, chunk.z)
-        f = self.folder.child(first).child(second)
-        if not f.exists():
-            f.makedirs()
-        f = f.child(filename)
-        write_to_file(chunk.save_to_tag(), f.open("w"))
+        self.serializer.save_chunk(chunk)
 
         chunk.dirty = False
 
@@ -334,69 +356,60 @@ class World(LevelSerializer):
         Retrieve player data.
         """
 
-        player = self.factory.create_entity(self.spawn[0], self.spawn[1],
-            self.spawn[2], "Player", username=username)
-
+        player = Player(username=username)
+        player.location.x = self.spawn[0]
+        player.location.y = self.spawn[1]
         player.location.stance = self.spawn[1]
-        player.username = username
+        player.location.z = self.spawn[2]
 
-        f = self.folder.child("players")
-        if not f.exists():
-            f.makedirs()
-        f = f.child("%s%s" % (username, extension()))
-        if f.exists() and f.getsize():
-            player.load_from_tag(read_from_file(f.open("r")))
+        self.serializer.load_player(player)
 
         return player
 
     def save_player(self, username, player):
+        if self.saving:
+            self.serializer.save_player(player)
 
-        f = self.folder.child("players")
-        if not f.exists():
-            f.makedirs()
-        f = f.child("%s%s" % (username, extension()))
-        write_to_file(player.save_to_tag(), f.open("w"))
+    # World-level geometry access.
+    # These methods let external API users refrain from going through the
+    # standard motions of looking up and loading chunk information.
 
-    def get_block(self, coords):
+    @coords_to_chunk
+    def get_block(self, chunk, coords):
         """
         Get a block from an unknown chunk.
         """
 
-        x, y, z = coords
+        return chunk.get_block(coords)
 
-        bigx, smallx, bigz, smallz = split_coords(x, z)
-        chunk = self.load_chunk(bigx, bigz)
-        return chunk.get_block((smallx, y, smallz))
-
-    def set_block(self, coords, value):
+    @coords_to_chunk
+    def set_block(self, chunk, coords, value):
         """
         Set a block in an unknown chunk.
         """
 
-        x, y, z = coords
+        chunk.set_block(coords, value)
 
-        bigx, smallx, bigz, smallz = split_coords(x, z)
-        chunk = self.load_chunk(bigx, bigz)
-        chunk.set_block((smallx, y, smallz), value)
-
-    def get_metadata(self, coords):
+    @coords_to_chunk
+    def get_metadata(self, chunk, coords):
         """
         Get a block's metadata from an unknown chunk.
         """
 
-        x, y, z = coords
+        return chunk.get_metadata(coords)
 
-        bigx, smallx, bigz, smallz = split_coords(x, z)
-        chunk = self.load_chunk(bigx, bigz)
-        return chunk.get_metadata((smallx, y, smallz))
-
-    def set_metadata(self, coords, value):
+    @coords_to_chunk
+    def set_metadata(self, chunk, coords, value):
         """
         Set a block's metadata in an unknown chunk.
         """
 
-        x, y, z = coords
+        chunk.set_metadata(coords, value)
 
-        bigx, smallx, bigz, smallz = split_coords(x, z)
-        chunk = self.load_chunk(bigx, bigz)
-        chunk.set_metadata((smallx, y, smallz), value)
+    @coords_to_chunk
+    def destroy(self, chunk, coords):
+        """
+        Destroy a block in an unknown chunk.
+        """
+
+        chunk.destroy(coords)
